// Implementazione del Client RMI

import java.rmi.*;
import java.io.*;

public class Client {

    // RMI Client start
    public static void main(String[] args) {
        int registryPort = 1099;
        String registryHost = null;
        String serviceName = "Server";
        BufferedReader stdIn = new BufferedReader(new InputStreamReader(System.in));

        // Check command line parameters
        if (args.length != 1 && args.length != 2) {
            System.out.println("Syntax: Client HostName [registryPort]");
            System.exit(1);
        } else {
            registryHost = args[0];
            if (args.length == 2) {
                try {
                    registryPort = Integer.parseInt(args[1]);
                } catch (Exception e) {
                    System.out.println("Syntax: Client HostName [registryPort]");
                    System.exit(2);
                }
            }
        }

        // Connection to RMI server
        try {
            String completeName = "//" + registryHost + ":" + registryPort + "/" + serviceName;
            RemOp serverRMI = (RemOp) Naming.lookup(completeName);
            System.out.println("RMI Client: Connected to service \"" + serviceName + "\"");

            System.out.println("\nService requests until EOF");

            String service;
            System.out.print("Service (C=count occurrences, L=list file names containing a character): ");

            while ((service = stdIn.readLine()) != null) {

                if (service.equals("C")) {

                    String fileName;
                    char c;
                    System.out.print("File name? ");
                    fileName = stdIn.readLine();
                    System.out.print("Character? ");
                    // c = first char of the line
                    c = stdIn.readLine().charAt(0);

                    // Remote invocation
                    try {
                        int occurrences = serverRMI.count_occurrences(fileName, c);
                        System.out.println("File \"" + fileName + " contains " + occurrences + " occurrences of character '" + c + "'.");
                    } catch (RemoteException re) {
                        System.out.println("Remote exception: " + re.toString());
                    }

                } // C
                else if (service.equals("L")) {
                    String dirName;
                    char c;
                    System.out.print("Directory name? ");
                    dirName = stdIn.readLine();
                    System.out.print("Character? ");
                    // c = first char of the line
                    c = stdIn.readLine().charAt(0);

                    String[] res;
                    try {
                        res = serverRMI.list_files_char(dirName, c);
                        System.out.println("File names:");
                        for (int i = 0; i < res.length; i++) {
                            System.out.println(res[i]);
                        }
                    } catch (RemoteException re) {
                        System.out.println("Remote exception: " + re.toString());
                    }
                } // L
                else {
                    System.out.println("Service unavailable.");
                }

                System.out.print("Service (C=count occurrences, L=list file names containing a character): ");
            } // !EOF

        } catch (Exception e) {
            System.err.println("RMI Client: " + e.getMessage());
            e.printStackTrace();
            System.exit(1);
        }
    }
}
