// Server Implementation

import java.io.File;
import java.io.FileInputStream;
import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

public class ServerImpl extends UnicastRemoteObject implements RemOp {

    // Constructor
    public ServerImpl() throws RemoteException {
        super();
    }

    // Count character occurrence in file "file"
    public int count_occurrences(String file, char c) throws RemoteException {
        int count = 0;
        File f = new File(file);
        if (!f.exists()) {
            throw new RemoteException("File \"" + file + "\" does not exist.");
        }
        FileInputStream is = null;
        try {
            is = new FileInputStream(file);
            // we will save "character" in buffer
            int buffer = 0;
            // exit from loop if read() returns a negative value -> EOF
            while ((buffer = is.read()) >= 0) {
                if (((char) buffer) == c) {
                    count++;
                }
            }
        } catch (Exception e) {
            throw new RemoteException(e.toString());
        }
        return count;
    }

    // Returns the list of files whose name contains character c
    public String[] list_files_char(String directory, char c)
            throws RemoteException {
        String[] temp;
        File f = new File(directory);
        if ((!f.exists()) || (!f.isDirectory())) {
            throw new RemoteException("Directory \"" + directory + "\" does not exist.");
        }
        temp = f.list();
        int filenamesToReturn = 0;
        // count how many filenames we have to return
        for (int i = 0; i < temp.length; i++) {
            if (temp[i].indexOf(c) != -1) {
                filenamesToReturn++;
            }
        }
        String[] result = new String[filenamesToReturn];
        // Copy filenames in result array
        int j = 0;
        for (int i = 0; i < temp.length; i++) {
            if (temp[i].indexOf(c) != -1) {
                result[j] = temp[i];
                j++;
            }
        }
        return result;
    }

    // RMI Server main
    public static void main(String[] args) {

        int registryPort = 1099;
        String registryHost = "localhost";
        String serviceName = "Server";

        // Check command line arguments
        if (args.length != 0 && args.length != 1) {
            System.out.println("Syntax: ServerImpl [registryPort]");
            System.exit(1);
        }
        if (args.length == 1) {
            try {
                registryPort = Integer.parseInt(args[0]);
            } catch (Exception e) {
                System.out.println("Syntax: ServerImpl [registryPort]");
                System.exit(2);
            }
        }

        // Register RMI service
        String completeName = "//" + registryHost + ":" + registryPort + "/"
                + serviceName;
        try {
            ServerImpl serverRMI = new ServerImpl();
            Naming.rebind(completeName, serverRMI);
            System.out.println("RMI Server: Service \"" + serviceName + "\" registered");
        } catch (Exception e) {
            System.err.println("RMI Server:\"" + serviceName + "\": " + e.getMessage());
            e.printStackTrace();
            System.exit(1);
        }
    }
}
